#include "ftpreader.h"
#include "cmdshell.h"
#include "sortfile.h"

using namespace MLSUTIL;
using namespace MLS;

FtpReader::FtpReader()
{
	_pDefaultFtpNet = NULL;
	_sReaderName = "ftp";
}

FtpReader::~FtpReader()
{
}

int		FtpReader::GetIpUserPw(const string& sStr, string& sIP, string& sUser, string& sPasswd)
{
	string sStrTmp = sStr;
	if (sStrTmp != "")
	{
		string::size_type p = sStrTmp.rfind("@");
		string::size_type p1 = sStrTmp.find(":");
		
		if (p != string::npos) 
		{
			sIP = sStrTmp.substr(p+1);
			if (p1 != string::npos)
			{
				sUser = sStrTmp.substr(0, p1);
				sPasswd = sStrTmp.substr(p1+1, p-p1-1);
			}
			else
			{
				sUser = sStrTmp.substr(0, p);
			}
		}
		else
		{
			sIP = sStrTmp;
		}
	}
	return SUCCESS;
}

int		FtpReader::LineFormatRead(vector<string>&	vLineToken, File* pFileInfo)
{
	// -rw-------   1 root  root       4223 Aug 14  2004 .test
	// -rwxr-xr-x   1 test  users       116 Jan  3  2005 Test
	// drwxr-xr-x   2 test  users      4096 May  4 11:50 Test1
	pFileInfo->sAttr = vLineToken[0];

	if(vLineToken.size() > 11 || vLineToken.size() < 9) return ERROR;
	
	if (pFileInfo->sAttr[0] == 'd' || pFileInfo->sAttr[0] == '-')
	{
		pFileInfo->sType = _sInitTypeName;		
		pFileInfo->uSize = atol(vLineToken[4].c_str());
		pFileInfo->sDate = vLineToken[5] + " " + vLineToken[6];
		pFileInfo->sTime = vLineToken[7];
		if (pFileInfo->sAttr[0] == 'd')
			pFileInfo->bDir = true;
		else
			pFileInfo->bDir = false;
		pFileInfo->sFullName = _sCurPath + vLineToken[8];
		pFileInfo->sName = vLineToken[8];
				
		// Link Setting	
		if (pFileInfo->sAttr[0] == 'l')
			if (vLineToken.size() == 11)
				pFileInfo->sName = pFileInfo->sName + " -> " + vLineToken[10];
	}
	else
		return ERROR;
	return SUCCESS;
}

void	FtpReader::Init(const string& sInitFile)
{
	string 	sIP, sUser, sPasswd;
	// sInitFile : username:passwd@IP
	// example   : annonymous:123123@name.com
	string sConnectionInfo = sInitFile;

	while(1)
	{
		GetIpUserPw(sConnectionInfo, sIP, sUser, sPasswd);
		if (sIP == "" && sUser == "" && sPasswd == "")
		{	
			if (InputBox(	gettext("Input ftp connect url (user:pswd@hostname)"),
							sConnectionInfo) == ERROR) return;
		}
		else
			break;
	}
	
	if (sUser == "" && sPasswd == "")
	{
		String sMsg;
		sMsg.Append("Input username - [%s]", sIP.c_str());
		if (InputBox(sMsg.c_str(), sUser) == ERROR) return;

		sMsg.clear();
		sMsg.Append("Input passwd - [%s@%s]", sUser.c_str(), sIP.c_str());
		if (InputBox(sMsg.c_str(), sPasswd, true) == ERROR) return;
	}
	else if (sPasswd == "")
	{
		String sMsg;
		sMsg.Append("Input passwd - [%s@%s]", sUser.c_str(), sIP.c_str());
		if (InputBox(sMsg.c_str(), sPasswd, true) == ERROR) return;
	}

	void*	pWait = MsgWaitBox(gettext("Ftp connect"), gettext("Please wait !!!"));

	if (FtpConnect(sIP.c_str(), &_pDefaultFtpNet) == 0)
	{
		String	sConnErrMsg;
		sConnErrMsg.Append("ftp connect fail !!! - %s", sIP.c_str());
		MsgWaitEnd(pWait);
		MsgBox(gettext("Error"), sConnErrMsg);
		return;
	}

	if (FtpLogin( sUser.c_str(), sPasswd.c_str(), _pDefaultFtpNet) == 0)
	{
		String	sLoginErrMsg;
		sLoginErrMsg.Append("ftp login fail !!! - %s", sUser.c_str());
		MsgWaitEnd(pWait);
		MsgBox(gettext("Error"), sLoginErrMsg);
		Destroy();
		return;
	}

	MsgWaitEnd(pWait);

	string sPath = GetPwd();
	if (sPath == "")
	{
		MsgBox(gettext("Error"), "get current dir reading failure !!!");
		Destroy();
		return;
	}
	
	_sCurPath = _sHome = sPath;
	_sInitTypeName = "ftp://" + sUser + "@" + sIP;
	_sIP = sIP; 
	_sUser = sUser;

	g_Log.Write("Ftp Connected [%s] [%s]", _sInitTypeName.c_str(), _sCurPath.c_str());

	if (Read(_sCurPath) == false)
	{
		Destroy();
	}
}

void	FtpReader::Destroy()
{		
	_sInitTypeName = "";
	_sCurPath = "";
	FtpQuit(_pDefaultFtpNet);
	_pDefaultFtpNet = NULL;

	string	sDeleteFile;
	string  sTmp = _sUser + "@" + _sIP;
	_vDeleteFile.push_back(sTmp);
	for (int n = 0; n < _vDeleteFile.size(); n++)
	{
		sDeleteFile = _vDeleteFile[n];
		sDeleteFile = "rm -rf /tmp/" + addslash(sDeleteFile) + " > /dev/null 2> /dev/null";
		system(sDeleteFile.c_str());
	}
	_vDeleteFile.clear();
	_sIP = ""; _sUser = "";
	_sHome = "";
}

string	FtpReader::GetPwd() const
{
	string sPathTmp;
	char*	cPath = new char[1024];	
	memset(cPath, 0, 1024);
	if (FtpPwd(cPath, 1024, _pDefaultFtpNet) == 0)
	{
		delete []cPath;
		return sPathTmp;
	}
	sPathTmp = sPathTmp + cPath + "/";
	delete []cPath;
	return sPathTmp;
}

/// \brief	상대경로는 절대경로로 바꾼다.
/// \param	str		상대경로 path명
///	\return	절대경로명
string FtpReader::GetRealPath(const string& str) const
{
	string sPath = str;
	
	// ~ 인지 먼저 판단.
	if (sPath[0]=='~')
	{			
		if (sPath == "~" || sPath[1] == '/')
		{
			// . home 디렉토리 지정
			string sTmp1 = sPath.substr(1);
			string sTmp2;
			if (sTmp1.find('/') != -1)
				sTmp2 = sTmp1.substr(sTmp1.find('/')+1);
			sPath = _sHome + sTmp2;
		}
		else
		{	
			sPath = str;
		}
	}
	else if (sPath[0]!='/') // .. /(root) dir이 아니면
	{
		// ... 현재 dir를 얻어야 한다면 현재 dir절대경로를 찾는다. 
		if (sPath == ".") sPath = _sCurPath.empty() ? GetPwd() : _sCurPath;
		// ... 상위 dir를 얻어야 한다면
		else if (sPath == "..")
		{
			// .... /(root)dir이 아니면 상위 절대경로 dir명을 구함
			if (_sCurPath != "/")
			{
				string::size_type p = _sCurPath.rfind('/', _sCurPath.size()-2);
				sPath = _sCurPath.substr(0, p+1);
			}			
			/// .... /(root)dir이면 절대경로에 / 를 넣음
			else sPath = _sCurPath;
		}
		else // ... 그외 dir은 path에 추가하여 절대경로를 구한다.
		{
			sPath = _sCurPath + sPath;
		}
	}
	if (sPath.substr(sPath.length()-1, 1) != "/") sPath += '/';
	return sPath;
}

bool	FtpReader::Read(const string &sPath)
{
	if (_pDefaultFtpNet == NULL) return false;

	string	sPathReal;
	if (sPath == "")
		sPathReal = _sCurPath;
	else
		sPathReal = GetRealPath(sPath);

	if (FtpChdir(sPathReal.c_str(), _pDefaultFtpNet) == 0)
	{
		String sMsgStr;
		sMsgStr.Append("access error - %s", sPathReal.c_str());
		MsgBox(gettext("Error"), sMsgStr.c_str());
		return false;
	}
	
	string sListFp = "/tmp/" + _sUser + "@" + _sIP;
	g_Log.Write("Read List File [%s] [%s]", sListFp.c_str(), sPathReal.c_str());
	if (FtpDir(sListFp.c_str(), sPathReal.c_str(), _pDefaultFtpNet) == 0)
	{
		String sMsgStr;
		sMsgStr.Append("access error - %s", sPathReal.c_str());
		MsgBox(gettext("Error"), sMsgStr.c_str());
		return false;
	}

	string sBefPath = _sCurPath;
	_sCurPath = sPathReal;
	
	vector<LineArgData> vLineArgData;
	if (CmdShell::CmdExeArg(sListFp, 0, vLineArgData, true) == ERROR) 
	{
		_sCurPath = sBefPath;
		return false;
	}

	g_Log.Write("CmdExeArg vLineArgData size [%d]", vLineArgData.size());

	vector<File*>	tTmpList;

	File*	pFileInfo = NULL;
	for (uint n = 0; n < vLineArgData.size(); n++)
	{
		pFileInfo = new File;
		string	sLine;
		for (uint c = 0; c < vLineArgData[n].size(); c++)
		{
			vector<string>&	tStr = vLineArgData[n];
			sLine = sLine + "[" + tStr[c] + "]";
		}
		g_Log.Write("Data [%s]", sLine.c_str());
		if (LineFormatRead(vLineArgData[n], pFileInfo) == SUCCESS)
			tTmpList.push_back(pFileInfo);
		else
		{
			delete pFileInfo; pFileInfo = NULL;
		}
	}

	for (uint n = 0; n < _vFileLists.size(); n++)
	{
		File* pFile = _vFileLists[n];
		delete pFile;
	}
	_vFileLists.clear();

	if (sPathReal != "/")
	{
		File*	pFile = new File;
		pFile->sType = _sInitTypeName;
		pFile->sAttr = "drwxr--r--";
		pFile->uSize = 0;
		pFile->bDir = true;
		pFile->sDate = "--------";
		pFile->sTime = "--:--";
		pFile->sFullName =  GetRealPath("..");
		pFile->sName = "..";
		tTmpList.push_back(pFile);
	}
	_vFileLists = tTmpList;
	_uCur = 0;
	return true;
}

bool	FtpReader::Next()
{
	if (_uCur < _vFileLists.size())
	{
		_uCur++;
		return true;
	}
	return false;
}

bool FtpReader::GetInfo(File &tFile)
{
	File*	pFile = NULL;

	// . 현재 선택된 파일이 유효하면
	if (_uCur-1 < _vFileLists.size())
	{
		// .. filelist에서 파일 정보를 가져옴
		pFile = _vFileLists[_uCur-1];
		tFile.Clear();

		if (pFile->sName == "." ) return false;
		tFile = *pFile;
	}
	else 
		return false;
	return true;
}

bool	FtpReader::Rename(File* pFile, const string& sRename)
{
	if (pFile == NULL) 
	{
		g_Log.Write("Rename pFile is NULL !!!");
		return false;
	}

	string sRenameName = pFile->sName;
	if (sRenameName == "..") return false;

	if (sRename == "")
	{
		if (InputBox(gettext("Rename"), sRenameName) == ERROR) 
			return false;
	}
	else
		sRenameName = sRename;
	
	sRenameName = _sCurPath + sRenameName;
	if (FtpRename(pFile->sFullName.c_str(), sRenameName.c_str(), _pDefaultFtpNet) == 0)
	{
		String sMsg;
		sMsg.Append("Rename failure !!! - %s", pFile->sName.c_str());
		MsgBox(	gettext("Error"), sMsg.c_str());
		return false;
	}
	return true;
}

bool	FtpReader::Mkdir(const string& sFullPathName)
{
	string sMkdirName;
	if (sFullPathName == "")
	{
		if (InputBox(gettext("Make Directory"), sMkdirName) == ERROR)
			return false;
	}
	else
		sMkdirName = sFullPathName;
	
	sMkdirName = _sCurPath + sMkdirName;
	if (FtpMkdir(sMkdirName.c_str(), _pDefaultFtpNet) == 0)
	{
		String sMsg;
		sMsg.Append("Mkdir failure !!! - %s", sMkdirName.c_str());
		MsgBox(	gettext("Error"), sMsg.c_str());
		return false;
	}
	return true;
}

///	\brief	파일을 보기 위한..
///	\param	tFile	볼 파일.
///	\return	파일을 tmp 에 복사 해놓고 파일 위치 리턴
bool  FtpReader::View(const File* pFileOriginal, File* pFileChange)
{ 
	string sTmpName = "/tmp/" + pFileOriginal->sName;
	if (FtpGet(	sTmpName.c_str(), pFileOriginal->sName.c_str(), 
				FTPLIB_IMAGE, _pDefaultFtpNet) == 0)
	{
		String 	sMsg;
		sMsg.Append("%s file recv failure !!!", pFileOriginal->sName.c_str());
		MsgBox("Error", sMsg);
		return false;
	}
	pFileChange->sFullName = sTmpName;
	pFileChange->sName = pFileOriginal->sName;
	_vDeleteFile.push_back(sTmpName);
	return true;
}

///	\brief	파일 복사
///	\return	복사 여부	
bool	FtpReader::Copy(	Selection& tSelection, 
							const string& sTargetPath,
							Selection*	pSelection)
{
	string			sCurName, sTargetPathTmp, sTargetPathTmp2;
	string			sSourceName, sTargetName;
	vector<File*>	vFiles;
	int				nViewCount, uPsize;

	uint	uFileSize = tSelection.CalcSize();
	uint	uSize = tSelection.GetSize();
	string 	sSize = toregular(uSize), sRestSize;

	vFiles = tSelection.GetData();

	if (sTargetPath.size() == 0)
		sTargetPathTmp2 = _sCurPath;
	else
		sTargetPathTmp2 = sTargetPath;

	ProgressBox	tProgress(	gettext("Copy"), (gettext("To ") + sTargetPathTmp2).c_str(), true);
	
	if (pSelection != NULL)
	{
		pSelection->Clear();
		pSelection->SetSelectPath(sTargetPathTmp2);
	}

	string sSourcePath = tSelection.GetSelectPath();
	
	g_Log.Write("FtpReader Copy sTargetPath [%s] [%s] [%s]", sTargetPath.c_str(), _sCurPath.c_str(), sTargetPathTmp2.c_str());

// 파일 복사
	for (int n=0; n<vFiles.size(); n++)
	{
		File*	pFile = vFiles[n];

		// 파일 이름이 없을경우 continue
		if (!pFile) 
		{
			g_Log.Write("FtpReader Copy pFile is NULL !!!");
			continue;
		}

		// 소스 이름과 타겟 이름 정하기	
		sSourceName = pFile->sFullName;
		sTargetName = sTargetPathTmp2 + pFile->sFullName.substr(sSourcePath.size());

		// 파일 리스트가 디렉토리 라면
		if (pFile->bDir)
		{
			// 타겟 디렉토리가 있는지 검사한다..
			struct stat dir_stat;

			if (pSelection != NULL)
			{
				File	tTargetFile = *pFile;
				tTargetFile.sFullName = sTargetName;
				pSelection->Add(&tTargetFile);
			}
			
			if (stat(sTargetName.c_str(), &dir_stat) != -1)
			{
				// 이미 디렉토리가 있다면
				continue;
			}
			else if (mkdir(sTargetName.c_str(), 0755)==-1) // 디렉토리를 만든다.
			{// 실패하면
				MsgBox(gettext("Error"), gettext("Directory Making error"));
				break;
			}
			continue;
		}

		String	sCount;
		sCount.Append("<%d/%ld>", nViewCount, vFiles.size());

		tProgress.setLeftStr(pFile->sName.c_str());
		tProgress.setRightStr(sSize.c_str());
		tProgress.setRightStr2(sCount.c_str());
		tProgress.redraw();

		nViewCount++;

		uPsize += pFile->uSize;
		sRestSize = toregular(uPsize);
		
		tProgress.setLeftStr(sRestSize.c_str());
		if (uPsize && uFileSize)
			tProgress.setCount(0, uPsize * 100 / uFileSize);
		tProgress.show();

		g_Log.Write("FtpReader Copy [%s] [%s]", sSourceName.c_str(), sTargetName.c_str());
		
		if (FtpGet(sTargetName.c_str(), sSourceName.c_str(), FTPLIB_IMAGE, _pDefaultFtpNet) == 0)
		{
			String 	sMsg;
			sMsg.Append("%s file get failure. : continue ?", pFile->sName.c_str());
			MsgBox("Error", sMsg);
			if (YNBox(gettext("Error"), sMsg.c_str(), false)==true)
				continue;
			else
				return false;
		}

		tProgress.show();

		if (pSelection != NULL)
		{
			File	tTargetFile = *pFile;
			tTargetFile.sFullName = sTargetName;
			pSelection->Add(&tTargetFile);
		}
	}
halt:
	if (pSelection)
		pSelection->SetSelectPath(sTargetPathTmp2);
	
	return true;
}

///	\brief	파일 삭제
///	\return	삭제 여부
bool	FtpReader::Remove(Selection&	tSelection, bool bMsgShow)
{
	ProgressBox tProgress(gettext("Delete"), "");

	if (bMsgShow)
	{
		if (YNBox(gettext("Info"), gettext("Delete ?"), false) == false) return 0;
		
		tProgress.setLeftStr(gettext("Counting files..."));
		tProgress.show();
	}

	uint	uSize = tSelection.GetSize();
	string	sTargetName;
	
	vector<string> dir_stack;	

	vector<File*>	vFiles;
	vFiles = tSelection.GetData();
	vector<File*>	vFile, vDirs;
	
	// 파일 이동
	for (int t=0; t< vFiles.size(); t++)
	{
		File*	pFile = vFiles[t];
		// 파일 이름이 없을경우 continue
		if (!pFile) continue;
		if (!pFile->bDir)
			vFile.push_back(pFile);
		else
			vDirs.push_back(pFile);
	}

	String	sStr;

	int 	nCount = 0;
	if (bMsgShow)
	{
		sStr.Printf("%d/%d %%", nCount, uSize);
		tProgress.setRightStr(sStr.c_str());
		if (nCount && uSize)
			tProgress.setCount((int)((long)(nCount/uSize)/100.0));
		tProgress.show();
	}
	
	// 파일 긴것부터 삭제. 
	vector<File *>::iterator   	begin = vFile.begin(),	end = vFile.end();
	sort(begin, end, sort_dir_adaptor<sort_fullname_length, sort_fullname_length>());
	begin = vDirs.begin();	end = vDirs.end();
	sort(begin, end, sort_dir_adaptor<sort_fullname_length, sort_fullname_length>());

	for (int n=0; n < vFile.size(); n++)
	{
		File*	pFile = vFile[n];
		sTargetName = pFile->sFullName;
		g_Log.Write("File Remove [%s]", sTargetName.c_str());

		if (FtpDelete(sTargetName.c_str(), _pDefaultFtpNet) == 0)
		{
			String	sStr;
			sStr.Append(gettext("ftp file remove failure : %s : continue ? "), pFile->sName.c_str());
			if (YNBox(gettext("Error"), sStr.c_str(), false)==true)
				continue;
			else
				return false;
			break;
		}
		nCount++;
		if (bMsgShow)
		{
			sStr.Printf("%d/%d %%", nCount, uSize);
			tProgress.setRightStr(pFile->sName.c_str());
			if (nCount && uSize)
				tProgress.setCount((int)(((double)nCount/(double)uSize)*100.0));
			tProgress.show();
		}
	}

	for (int n=0; n < vDirs.size(); n++)
	{
		File*	pFile = vDirs[n];
		sTargetName = pFile->sFullName;
		g_Log.Write("Dir Remove [%s]", sTargetName.c_str());
		if (FtpDelete(sTargetName.c_str(), _pDefaultFtpNet) == 0)
		{
			String	sStr;
			sStr.Append(gettext("ftp dir remove failure %s : continue ?"), 
					sTargetName.c_str());
			if (YNBox(gettext("Error"), sStr.c_str(), false)==true)
				continue;
			else
				return false;
			break;
		}

		nCount++;
		if (bMsgShow)
		{
			sStr.Printf("%d/%d %%", nCount, uSize);
			tProgress.setRightStr(sTargetName.c_str());
			if (nCount && uSize)
			{
				tProgress.setCount((int)(((double)nCount/(double)uSize)*100.0));
			}
			tProgress.show();
		}
	}
	return true;
}

bool	FtpReader::Paste(Selection& tSelection)
{
	string			sCurName, sTargetPathTmp, sTargetPathTmp2;
	string			sSourceName, sTargetName;
	vector<File*>	vFiles;
	int	uPsize, nViewCount;

	uint	uFileSize = tSelection.CalcSize();
	uint	uSize = tSelection.GetSize();
	
	string 	sSize = toregular(uSize), sRestSize;

	vFiles = tSelection.GetData();
	sTargetPathTmp2 = _sCurPath;
	
	ProgressBox	tProgress(	gettext("Copy"), (gettext("To ") + sTargetPathTmp2).c_str(), true);
	
	string sSourcePath = tSelection.GetSelectPath();
	
	g_Log.Write("Copy sTargetPath [%s] [%s]", _sCurPath.c_str(), sTargetPathTmp2.c_str());

	// 파일 디렉토리 먼저 생성 복사
	for (int n=0; n<vFiles.size(); n++)
	{
		File*	pFile = vFiles[n];
		
		// 파일 이름이 없을경우 continue
		if (!pFile) continue;
		// 소스 이름과 타겟 이름 정하기	
		sSourceName = pFile->sFullName;
		sTargetName = sTargetPathTmp2 + pFile->sFullName.substr(sSourcePath.size());

		// 파일 리스트가 디렉토리 라면
		if (pFile->bDir)
		{
			if (FtpMkdir(sTargetName.c_str(), _pDefaultFtpNet) == 0)
			{
				// 이미 디렉토리가 있다면
				if (YNBox(gettext("Error"), gettext("Directory exists, continue?"), false) == false) break;
				continue;
			}
			continue;
		}
	}

	for (int n=0; n<vFiles.size(); n++)
	{
		File*	pFile = vFiles[n];
		
		// 파일 이름이 없을경우 continue
		if (!pFile) continue;

		// 소스 이름과 타겟 이름 정하기	
		sSourceName = pFile->sFullName;
		sTargetName = sTargetPathTmp2 + pFile->sFullName.substr(sSourcePath.size());

		g_Log.Write("FtpReader::Paste [%s] [%s]", sSourceName.c_str(), sTargetName.c_str());

		// 파일 리스트가 디렉토리 라면
		if (pFile->bDir)
		{
			continue;
		}

		String	sCount;
		sCount.Append("<%ld/%ld>", nViewCount, vFiles.size());

		tProgress.setLeftStr(pFile->sName.c_str());
		tProgress.setRightStr(sSize.c_str());
		tProgress.setRightStr2(sCount.c_str());
		tProgress.redraw();

		nViewCount++;

		uPsize += pFile->uSize;
		sRestSize = toregular(uPsize);
		
		tProgress.setLeftStr(sRestSize.c_str());
		if (uPsize && uFileSize)
		{
			tProgress.setCount(0, uPsize * 100 / uFileSize);
		}
		tProgress.show();
		
		if (FtpPut(sSourceName.c_str(), sTargetName.c_str(), FTPLIB_IMAGE, _pDefaultFtpNet) == 0)
		{
			String 	sMsg;
			sMsg.Append("%s file put failure. : continue ?", pFile->sName.c_str());
			if (YNBox(gettext("Error"), sMsg.c_str(), false)==true)
				continue;
			else
				return false;
		}

		tProgress.show();
	}
halt:
	return true;
}
