#include "command.h"
#include "commandimp.h"
#include "mlsdialog.h"
#include "ncurses_panel.h"

namespace MLS
{

Command::Command()
{
	_pCmdImp = new CommandImp;
	Init();
}

Command::~Command()
{
	delete _pCmdImp;
}

void	Command::Init()
{
	// -- Key
	_mapFunc["Left"]		= &CommandImp::Key_Left;
	_mapFunc["Right"]		= &CommandImp::Key_Right;
	_mapFunc["Up"]			= &CommandImp::Key_Up;
	_mapFunc["Down"]		= &CommandImp::Key_Down;
	_mapFunc["Home"]		= &CommandImp::Key_Home;
	_mapFunc["End"]			= &CommandImp::Key_End;
	_mapFunc["PgUp"]		= &CommandImp::Key_PageUp;
	_mapFunc["PgDn"]		= &CommandImp::Key_PageDown;
	_mapFunc["Enter"]		= &CommandImp::Key_Enter;
	_mapFunc["Select"]		= &CommandImp::Key_Select;

	_mapFunc["GoRoot"] 		= &CommandImp::GoRoot;
	_mapFunc["GoHome"] 		= &CommandImp::GoHome;
	_mapFunc["GoParent"] 	= &CommandImp::GoParent;

	_mapFunc["SelectInvert"] = &CommandImp::SelectInvert;
	_mapFunc["SelectAll"] 	= &CommandImp::SelectAll;
	
	_mapFunc["View"]		= &CommandImp::View;
	_mapFunc["Edit"]		= &CommandImp::Editor;
	_mapFunc["ExtEdit"]		= &CommandImp::ExtEditor;

	_mapFunc["Menu"]		= &CommandImp::Menu;

	_mapFunc["Refresh"]	 	= &CommandImp::Refresh;

	_mapFunc["ViewSync"]	= &CommandImp::ViewSync;
	_mapFunc["SplitViewSync"] = &CommandImp::SplitViewSync;
	_mapFunc["Split"]		= &CommandImp::Split;
	_mapFunc["NextWindow"]	= &CommandImp::NextWindow;
	_mapFunc["Quit"]		= &CommandImp::Quit;

	_mapFunc["Archive"]		= &CommandImp::ArchiveFileView;

	_mapFunc["CopyClip"]	= &CommandImp::CopyClip;
	_mapFunc["CutClip"]		= &CommandImp::CutClip;
	_mapFunc["PasteClip"]	= &CommandImp::PasteClip;
	_mapFunc["Remove"]		= &CommandImp::Remove;
	_mapFunc["Rename"]		= &CommandImp::Rename;
	_mapFunc["Mkdir"]		= &CommandImp::Mkdir;

	_mapFunc["FtpConn"]		= &CommandImp::FtpConn;
	_mapFunc["FtpClose"]	= &CommandImp::FtpClose;
	_mapFunc["ESC"]			= &CommandImp::Key_ESC;
	_mapFunc["Shell"]		= &CommandImp::Shell;

	_mapFunc["BoxCodeChange"] = &CommandImp::BoxCodeChange;
	_mapFunc["LangChange"] = &CommandImp::LangChange;

	_mapFunc["HiddenFileView"] = &CommandImp::HiddenFileView;
	_mapFunc["SortChange"] 	= &CommandImp::SortChange;

	_mapFunc["MCD"]			= &CommandImp::Mcd;
	_mapFunc["MCD_Quit"]	= &CommandImp::Key_ESC;

	_mapFunc["MCD_Left"]	= &CommandImp::Key_Left;
	_mapFunc["MCD_Right"]	= &CommandImp::Key_Right;
	_mapFunc["MCD_Up"]		= &CommandImp::Key_Up;
	_mapFunc["MCD_Down"]	= &CommandImp::Key_Down;
	_mapFunc["MCD_Home"]	= &CommandImp::Key_Home;
	_mapFunc["MCD_End"]		= &CommandImp::Key_End;
	_mapFunc["MCD_PgUp"]	= &CommandImp::Key_PageUp;
	_mapFunc["MCD_PgDn"]	= &CommandImp::Key_PageDown;
	_mapFunc["MCD_Enter"]	= &CommandImp::Key_Enter;
	_mapFunc["MCD_Mkdir"]	= &CommandImp::Mkdir;
	_mapFunc["MCD_Rename"]	= &CommandImp::Rename;
	_mapFunc["MCD_Remove"]	= &CommandImp::Remove;

	_mapFunc["MCD_SubDirOneSearch"] = &CommandImp::McdSubDirOneSearch;
	_mapFunc["MCD_SubDirAllSearch"] = &CommandImp::McdSubDirAllSearch;
	_mapFunc["MCD_SubDirClear"]     = &CommandImp::McdSubDirClear;

	_mapFunc["ColumnAuto"]  = &CommandImp::ColumnAuto;
	_mapFunc["Column1"]     = &CommandImp::Column1;
	_mapFunc["Column2"]     = &CommandImp::Column2;
	_mapFunc["Column3"]     = &CommandImp::Column3;
	_mapFunc["Column4"]     = &CommandImp::Column4;
}

void Command::SetPanel(NCurses_Panel *p)
{
	_pCmdImp->SetPanel(p);
}

void Command::SetMcd(NCurses_Mcd* p)
{
	_pCmdImp->SetMcd(p);
}

NCurses_Panel* Command::GetPanel()
{
	return _pCmdImp->GetPanel();
}

/// @brief	해당 str 명령을 실행시킨다.
/// @return	0
int Command::Execute(
	const std::string &		str		///< 실행할 명령어
	)
{
	if (str.empty()) return 0;
	if (str.substr(0, 4) == "Cmd_")
	{	
		string sCmd = str.substr(4);
		sCmd = chop(sCmd);
		
		FuncMap::iterator i;
		if ((i = _mapFunc.find(sCmd)) != _mapFunc.end())
		{
			(_pCmdImp->*((*i).second))();
		}
		else
		{
			String sMsg;
			sMsg.Append(gettext("configure command not found.. [%s]"), sCmd.c_str());
			MsgBox(gettext("Error"), sMsg.c_str());
		}
	}
	return 0;
}

int Command::Run( const std::string &str, bool bPause)
{
	_pCmdImp->Run(str, bPause);
}

int	 Command::ParseAndRun(const std::string &sCmd, bool bPause)
{
	_pCmdImp->ParseAndRun(sCmd, bPause);
}

};
