#include "commandimp.h"
#include "drawset.h"
#include "mainframe.h"
#include "selection.h"
#include "mlsmenu.h"
#include "archive.h"

using namespace MLS;
using namespace MLSUTIL;

void CommandImp::UpdateConfig()
{
}

void CommandImp::SaveConfig()
{
}

///	@brief	명령어를 실행한다.
///	@param	cmd		명령어및 인자
///	@param	bPause	명령어를 실행시키고 종료시 잠시 정지시킬지 여부
/// @param	bFork	Fork할 것인지?
///	@return	명령어 실행 결과
int CommandImp::Run( const std::string &cmd, bool bPause, bool bBackground)
{
	// command 창에서 exit, quit 를 치면 종료하게 한다.
	if (cmd == "exit" || cmd == "quit") 
	{
		Quit();
		return 0;
	}

	CursesDestroy();
	int status;

	if (bBackground)
	{
		cout << "mls $ " << cmd.c_str() << endl;
		string sCmd = cmd + " > /dev/null 2>&1 &";
		status = system(sCmd.c_str());
	}
	else
	{
		cout << "mls $ " << cmd.c_str() << endl;
		status = system(cmd.c_str());
	}

	if (bPause)
	{
		cbreak();
		noecho();

		cout << endl << gettext("Press any key to return Mls...") << '\r' << endl;
		getch();
	}

	CursesInit();
	return 0;
}

///	@brief	모든 커맨드 메시지를 관리
///	@param	p	실행 시킬 명령어
///	@return		명령어 실행 결과
int CommandImp::ParseAndRun(const string &p, bool Pause)
{
	bool	bFlag  = false,
		 	bPause = Pause,
		 	bParam = false,
		 	bMcd   = false ,
		 	bConfirm = false,
			bBackground = false,
			bRootExe = false;
			
	// cd 구현
	if (p.substr(0, 2) == "cd")
	{
		string sStr;
		if (p == "cd" || p == "cd ")
			sStr = _pPanel->GetReader()->GetRealPath("~");
		else
			sStr = _pPanel->GetReader()->GetRealPath(p.substr(3));
		sStr = Replace(sStr, "\\", "");
		if (sStr != "")	
		{
			if (_pPanel->Read(sStr) == true) 
			{
				if (g_tMainFrame.GetScreenSync())
					ViewSync();
				return 1;
			}
		}
		return 0;
	}
	
	string 	arg, tmp_name;

	File	tFile;
	
	// 파일을 복사 해서 보여주기 위한...
	{
		File* 	pFile = _pPanel->GetCurFile();
		if (_pPanel->GetReader()->View(pFile, &tFile) == false)
		{
			//MsgBox(	gettext("Error"), gettext("file reading failure !!!!"));
			return 0;
		}
	}
	
	// 파싱
	for (string::const_iterator i = p.begin(); i!= p.end(); i++)
	{
		if (*i == '%')
		{
			bFlag = true;
			continue;
		}
		
		if (bFlag)
		{
			switch(*i)
			{
			// 파일을 리턴해줌
			case '1':
			case 'F':
				arg += addslash(tFile.sFullName);
				break;
				
			case 'N':
			{
				if (tFile.sName[0]=='.') return 0;
				
				string::size_type p = tFile.sName.rfind('.');
				
				if (p != string::npos)
					arg += addslash(tFile.sFullName.substr(0, p-1));
				else
					arg += addslash(tFile.sFullName);
				break;
			}
			
			case 'E':
				if (tFile.Ext().empty()) return 0;
				arg += addslash(tFile.Ext());
				break;
				
			case 'W':
				bPause = true;
				break;
				
			case '%':
				arg += '%';
				break;
				
			case 'S':
			{
				Selection tSelection;
				_pPanel->GetSelection(tSelection);
				
				if (tSelection.Size() == 0) return 0;				
				bool first = true;				
			
				for (int t = 0; t< tSelection.Size(); t++)
				{
					if (first) first= false;
					else arg += ' ';
					arg += addslash(tSelection[t]->sFullName);
				}
				break;
			}
			
			case 's':
			{
				Selection tSelection;
				_pPanel->GetSelection(tSelection);
				
				if (!tSelection.Size() == 0)
				{
					bool first = true;					
				
					for (int t = 0; t< tSelection.Size(); t++)
					{
						if (first) first= false;
						else arg += ' ';
					
						arg += addslash(tSelection[t]->sFullName);
					}
				}
				else 
				{
					if (tFile.sName == "..") return 0;
					arg += addslash(tFile.sFullName);
				}
				break;
			}
			
			case 'd':			
				arg += addslash(_pPanel->GetPath());
				break;		
			
			case 'D':
				bMcd = true;
				break;
			
			case 'P':
				bParam= true;
				break;

			case 'R': // root 계정 실행시
				bRootExe = true;
				break;
				
			case 'H':
				arg += addslash(_pPanel->GetReader()->GetRealPath("~"));
				break;
				
			case 'Q':
				bConfirm = true;
				break;
				
			case 'B':
				bBackground = true;
				break;
				
			default:
				break;
			}
			
			bFlag = false;
			continue;
		}
		else arg += *i;
	}
	
	if (bMcd)
	{
		/*
		Mcd mcd(_tPanel);
		mcd.setCur(_tPanel->GetCurrentPath());
		if (!mcd.proc())
            return 0;
		arg += addslash(mcd.getCur());
		*/
	}
	
	if (bParam)	if (InputBox(gettext("Edit Parameter"), arg)<0) return 0;
		
	if (bConfirm)
		if (YNBox(gettext("Do you want to run this operation?"), false) != true) return 0;

	if (bRootExe)
		arg = "su - --command=\"" + arg + "\"";
	
	// 실행
	int nRt = Run(arg, bPause, bBackground);
	return 0;
}

void	CommandImp::SplitViewSync()
{
	g_tMainFrame.SetScreenSync(!g_tMainFrame.GetScreenSync());

	if (g_tMainFrame.GetScreenSync())
	{
		NCurses_Panel* pPanel = g_tMainFrame.GetPanel(1);
		NCurses_Mcd* pMcd = g_tMainFrame.GetMcd(0);
		
		void*	pWait = MsgWaitBox(	gettext("Wait"),
									gettext("Please wait !!! - Cancel Key [ESC]"));
		pMcd->Destroy();
		pMcd->SetReader(pPanel->GetReader());
		pMcd->Rescan(2);
		pMcd->AddDirectory(pPanel->GetReader()->GetPath());
		pMcd->SetCur(pPanel->GetReader()->GetPath());
		MsgWaitEnd(pWait);
		ViewSync();
	}
}

void	CommandImp::ViewSync()
{
	int nActive = g_tMainFrame.GetActiveNum();

	NCurses_Panel* pPanel = g_tMainFrame.GetPanel(1);
	NCurses_Mcd* pMcd = g_tMainFrame.GetMcd(0);
	
	switch (g_tMainFrame.GetActiveViewType())
	{
		case MCD:
		{
			pPanel->Read(pMcd->GetCurName());
			pMcd->_bFocus = true;
			pPanel->_bFocus = false;
			break;
		}

		case PANEL:
		{
			pMcd->AddDirectory(pPanel->GetReader()->GetPath());
			pMcd->SetCur(pPanel->GetReader()->GetPath());
			pPanel->_bFocus = true;
			pMcd->_bFocus = true;
			break;
		}
	}
	g_tMainFrame.Refresh(false);
}

void CommandImp::Key_Enter()
{
	if (g_tMainFrame.GetScreenSync())
	{
		NCurses_Panel* pPanel = g_tMainFrame.GetPanel(1);
		NCurses_Mcd* pMcd = g_tMainFrame.GetMcd(0);
	
		switch (g_tMainFrame.GetActiveViewType())
		{
			case PANEL:
			{
				Reader*  pBefReader = pPanel->GetReader();
				pPanel->Key_Enter();
				
				if (pPanel->GetReader() != pBefReader)
				{
					pMcd->Destroy();
					pMcd->SetReader(pPanel->GetReader());
					pMcd->Rescan(2);
					pMcd->AddDirectory(pPanel->GetReader()->GetPath());
					pMcd->SetCur(pPanel->GetReader()->GetPath());
					pMcd->_bFocus = true;
				}
				else
				{
					pMcd->AddDirectory(pPanel->GetReader()->GetPath());
					pMcd->SetCur(pPanel->GetReader()->GetPath());
				}
				break;
			}
	
			case MCD:
			{
				pPanel->Read(pMcd->GetCurName());
				break;
			}
		}
		g_tMainFrame.Refresh(false);
	}
	else
	{
		switch (g_tMainFrame.GetActiveViewType())
		{
			case PANEL:
			{
				_pPanel->Key_Enter();
				break;
			}
			case MCD:
			{
				_pPanel->Read(_pMcd->GetCurName());
				g_tMainFrame.SetActiveViewType(PANEL);
				Refresh();
				break;
			}
		}
	}
}

void CommandImp::Key_Left()
{ 
	switch (g_tMainFrame.GetActiveViewType())
	{
		case PANEL: _pPanel->Key_Left(); 	break;
		case MCD:	_pMcd->Key_Left(); 		break;
	}

	if (g_tMainFrame.GetScreenSync()) ViewSync();
}

void CommandImp::Key_Right() 		
{ 
	switch (g_tMainFrame.GetActiveViewType())
	{
		case PANEL: _pPanel->Key_Right(); 	break;
		case MCD:	_pMcd->Key_Right(); 	break;
	}
	if (g_tMainFrame.GetScreenSync()) ViewSync();
}

void CommandImp::Key_Up()
{ 
	switch (g_tMainFrame.GetActiveViewType())
	{
		case PANEL: _pPanel->Key_Up();	 	break;
		case MCD:	_pMcd->Key_Up();	 	break;
	}
	if (g_tMainFrame.GetScreenSync()) ViewSync();
}

void CommandImp::Key_Down()
{ 
	switch (g_tMainFrame.GetActiveViewType())
	{
		case PANEL: _pPanel->Key_Down();	break;
		case MCD:	_pMcd->Key_Down();	 	break;
	}
	if (g_tMainFrame.GetScreenSync()) ViewSync();
}

void CommandImp::Key_Home()
{ 
	switch (g_tMainFrame.GetActiveViewType())
	{
		case PANEL: _pPanel->Key_Home();	break;
		case MCD:	_pMcd->Key_Home();	 	break;
	}
	if (g_tMainFrame.GetScreenSync()) ViewSync();
}

void CommandImp::Key_End() 		
{ 
	switch (g_tMainFrame.GetActiveViewType())
	{
		case PANEL: _pPanel->Key_End();		break;
		case MCD:	_pMcd->Key_End();	 	break;
	}
	if (g_tMainFrame.GetScreenSync()) ViewSync();
}

void CommandImp::Key_PageUp()
{ 
	switch (g_tMainFrame.GetActiveViewType())
	{
		case PANEL: _pPanel->Key_PageUp();		break;
		case MCD:	_pMcd->Key_PageUp();	 	break;
	}
	if (g_tMainFrame.GetScreenSync()) ViewSync();
}

void CommandImp::Key_PageDown()
{
	switch (g_tMainFrame.GetActiveViewType())
	{
		case PANEL: _pPanel->Key_PageDown();	break;
		case MCD:	_pMcd->Key_PageDown();	 	break;
	}
	if (g_tMainFrame.GetScreenSync()) ViewSync();
}

void	CommandImp::Mcd()
{
	g_tMainFrame.SetScreenSync(false);

	void*	pWait = MsgWaitBox(	gettext("Wait"),
								gettext("Please wait !!! - Cancel Key [ESC]"));
	_pMcd->Destroy();
	_pMcd->SetReader(_pPanel->GetReader());
	_pMcd->Rescan(2);
	_pMcd->AddDirectory(_pPanel->GetReader()->GetPath());
	_pMcd->SetCur(_pPanel->GetReader()->GetPath());
	g_tMainFrame.SetActiveViewType(MCD);
	MsgWaitEnd(pWait);
	_pMcd->_bFocus = true;
}

void CommandImp::View()
{
	if (g_tMainFrame.GetActiveViewType() != PANEL) return;

	if (g_tCfg.GetValue("Default", "ExtViewer").empty())
	{
		EditorChoice(false);
	}
	else
	{
		EditorChoice(false, g_tCfg.GetValue("Default", "ExtViewer"));
	}
}

void CommandImp::Editor()
{
	if (g_tMainFrame.GetActiveViewType() != PANEL) return;

	if (g_tCfg.GetValue("Default", "ExtEditor").empty())
	{
		EditorChoice(false);
	}
	else
	{
		EditorChoice(false, g_tCfg.GetValue("Default", "ExtEditor"));
	}
}

void CommandImp::ExtEditor()
{
	if (g_tMainFrame.GetActiveViewType() != PANEL) return;

	if (g_tCfg.GetValue("Default", "ExtEditor").empty())
		EditorChoice(false, "vi");
	else
		EditorChoice(false, g_tCfg.GetValue("Default", "ExtEditor"));
}

void CommandImp::EditorChoice(bool bReadOnly, const string& sEditorCmd)
{
	File*	pFile = _pPanel->GetCurFile();
	if (!pFile) return;

	File 	tFile;
	if (_pPanel->GetReader()->View(pFile, &tFile) == false) return;

	bool bExtEditor = false;
	if (!sEditorCmd.empty()) bExtEditor = true;

	g_Log.Write("ParseAndRun !!! [%s]", sEditorCmd.c_str());

	if (sEditorCmd.empty())
		ParseAndRun("vi " + tFile.sFullName);
	else
		ParseAndRun(sEditorCmd + " " + tFile.sFullName);
	return;
}

void CommandImp::Refresh()
{
	g_tMainFrame.GetPanel(0)->Read(g_tMainFrame.GetPanel(0)->GetPath());
	g_tMainFrame.GetPanel(1)->Read(g_tMainFrame.GetPanel(1)->GetPath());

	g_tMainFrame.Refresh();
}

void CommandImp::Menu()
{
	MlsMenu		tMlsMenu;
	vector<string>	vDisableList;

	if (_pPanel->GetReader()->GetReaderName() == "file")
	{
		vDisableList.push_back("Cmd_FtpClose");
	}
	else if (_pPanel->GetReader()->GetReaderName() == "ftp")
	{
		vDisableList.push_back("Cmd_FtpConn");
	}
	else
	{
		vDisableList.push_back("Cmd_FtpConn");
		vDisableList.push_back("Cmd_FtpClose");
	}

	tMlsMenu.SetDisable(vDisableList);
	tMlsMenu.Create();
	tMlsMenu.Do();

	g_tMainFrame.Refresh(false);
	if (tMlsMenu.GetCurCmd().size() != 0)
	{
		g_Log.Write("Menu Run [%s]", tMlsMenu.GetCurCmd().c_str());
		g_tMainFrame.GetCommand()->Execute(tMlsMenu.GetCurCmd());
	}
}

void CommandImp::Quit()
{
	switch (g_tMainFrame.GetActiveViewType())
	{
		case MCD:
			if (!g_tMainFrame.GetScreenSync())
			{
				g_tMainFrame.SetActiveViewType(PANEL);
				Refresh();
				break;
			}
		case PANEL:
			if (g_tCfg.GetBool("Default", "AskMlsExit") == true)
			{
				bool bYN = YNBox(gettext("Mls Quit"), gettext("Do you really want to quit the mls ?"), true);
				if (bYN == false) return;
			}
			
			// 마지막 mls 가 마지막으로 사용한 디렉토리를 저장한다.
			if (_pPanel->GetReader()->GetInitType() == "file://")
			{
				FILE *fp= fopen((g_tCfg.GetValue("CfgHome") + "path").c_str(), "wb");
		
				if (fp)
				{
					fputs(_pPanel->GetPath().c_str(), fp);
					fclose(fp);
				}
			}
			g_tMainFrame.Exit();
			break;
	}
}

void CommandImp::Split()
{
	g_tMainFrame.Split();
}

void CommandImp::NextWindow()
{
	g_tMainFrame.NextWindow();
}

void CommandImp::ArchiveFileView()
{
	if (g_tMainFrame.GetActiveViewType() != PANEL) return;

	File* pFile = _pPanel->GetCurFile();

	g_Log.Write("ArchiveFileView [%s]", pFile->sFullName.c_str());
	if (pFile->sType.substr(0, 7) == "archive")
		MsgBox(gettext("Error"), "Archive file view failure !!!");
	else
	{
		if (_pPanel->GetReader()->GetReaderName() != "file")
			_pPanel->PluginOpen("/tmp/" + pFile->sName, "archive");
		else
			_pPanel->PluginOpen(pFile->sFullName, "archive");
	}
}

void CommandImp::ArchiveFileMake()
{
	if (g_tMainFrame.GetActiveViewType() != PANEL) return;	
}

void CommandImp::FtpConn()
{
	if (g_tMainFrame.GetActiveViewType() != PANEL) return;

	_pPanel->PluginOpen("", "ftp");
	g_tMainFrame.Refresh();
}

void CommandImp::FtpClose()
{
	if (g_tMainFrame.GetActiveViewType() != PANEL) return;

	_pPanel->PluginClose("file");
	Refresh();
}

void CommandImp::Remove()
{
	switch(g_tMainFrame.GetActiveViewType())
	{
	case MCD:
		_pPanel->Read(_pMcd->GetCurName());
		g_tMainFrame.SetActiveViewType(PANEL);
		break;
	case PANEL:
		_tSelection.Clear();
		_pPanel->GetSelection(_tSelection);
		_tSelection.ExpandDir(_pPanel->GetReader(), false);
		_pPanel->GetReader()->Remove(_tSelection);
		Refresh();
		break;
	}
}

void CommandImp::CopyClip()
{
	if (g_tMainFrame.GetActiveViewType() != PANEL) return;

	_tSelection.Clear();
	_pPanel->GetSelection(_tSelection);
	_tSelection.SetSelectPath(_pPanel->GetReader()->GetPath());
	_tSelection.ExpandDir(_pPanel->GetReader(), false);
	if (_tSelection.Size() == 0)
		_tClipboard.Set(CLIP_NONE);
	else
		_tClipboard.Set(CLIP_COPY);

	for (int n = 0; n < _tSelection.Size(); n++)
	{
		File*	pFile = _tSelection[n];

		g_Log.Write("CopyClip [%d] [%s]", n, pFile->sFullName.c_str());
	}
}

void CommandImp::CutClip()
{
	if (g_tMainFrame.GetActiveViewType() != PANEL) return;

	_tSelection.Clear();
	_pPanel->GetSelection(_tSelection);
	_tSelection.SetSelectPath(_pPanel->GetReader()->GetPath());
	_tSelection.ExpandDir(_pPanel->GetReader(), false);

	if (_tSelection.Size() == 0)
		_tClipboard.Set(CLIP_NONE);
	else
		_tClipboard.Set(CLIP_CUT);

	for (int n = 0; n < _tSelection.Size(); n++)
	{
		File*	pFile = _tSelection[n];

		g_Log.Write("CutClip [%d] [%s]", n, pFile->sFullName.c_str());
	}
}

void CommandImp::PasteClip()
{
	if (g_tMainFrame.GetActiveViewType() != PANEL) return;

	ClipState state = _tClipboard.GetState();

	if (state == CLIP_NONE) return;

	if (state == CLIP_COPY)
	{
		if (_tSelection.Size() > 0)
		{
			Selection	tSelectionCopy;

			g_Log.Write("PasteClip Copy Start 1 - [%d] [%s] [%s] !!!", 
								_tSelection.Size(),
								_tSelection[0]->sType.c_str(), 
								_pPanel->GetReader()->GetInitType().c_str());

			// Source 나 Target이 file이면 그대로 이용한다.
			if (_tSelection[0]->sType == "file://" && _pPanel->GetReader()->GetInitType() == "file://")
			{
				// /tmp 에 복사하고 나서 옮긴다.
				g_Log.Write("PasteClip Copy Start !!!");
				_pPanel->GetReader()->Copy(_tSelection);
				g_Log.Write("PasteClip Copy End !!!");
			}
			else
			{
				bool	bUseGlobalReaderCtl = false;
	
				// 패널이 두개이기 때문에 mainframe 의 panel에서 Type에 맞는 데이터를 읽어 온다.
				Reader* pReader = g_tMainFrame.GetPanel(0)->GetReader(_tSelection[0]->sType);
				if (pReader == NULL || pReader->GetInitType() != _tSelection[0]->sType)
				{
					pReader = g_tMainFrame.GetPanel(1)->GetReader(_tSelection[0]->sType);
					
					if (pReader == NULL || pReader->GetInitType() != _tSelection[0]->sType)
					{
						// 그래도 없으면 만들서 넣어야 함.
						string::size_type p = _tSelection[0]->sType.find("://");
						string sInitName;

						if (p == string::npos)
							sInitName = _tSelection[0]->sType;
						else
							sInitName = _tSelection[0]->sType.substr(0, p);
	
						// 이때는 g_tReaderCtl로 읽어 들여옴.
						pReader = g_tReaderCtl.Get(sInitName);
						if (pReader == NULL)
						{
							MsgBox(gettext("Error"), gettext("plugin not found."));
							return;
						}

						g_Log.Write("g_tReaderCtl USE [%p] [%s]", pReader, sInitName.c_str());

						bUseGlobalReaderCtl = true;	
						if (p == string::npos)
							sInitName = _tSelection[0]->sType;
						else
							sInitName = _tSelection[0]->sType.substr(p+3);

						pReader->Init(sInitName);
						g_Log.Write("g_tReaderCtl Init [%p] [%s]", pReader, sInitName.c_str());
					}
				}

				g_Log.Write("File Copy !!!");

				// /tmp 에 복사하고 나서 옮긴다.
				if (pReader->Copy(_tSelection, "/tmp/", &tSelectionCopy))
				{
					g_Log.Write("File Copy Complite !!!");
				
					if (_pPanel->GetReader()->GetInitType() == "file://")
					{
						g_Log.Write("File Copy Move !!!");

						// 파일일 경우는 이동한다.
						_pPanel->GetReader()->Move(tSelectionCopy);
					}
					else
					{	
						g_Log.Write("File Copy /tmp Remove !!!");

						// 파일이 아닌 경우는 /tmp에서 다른 곳으로 복사 한 후
						// /tmp에 있는 내용은 지운다.
						if (_pPanel->GetReader()->Paste(tSelectionCopy))
							if ((pReader=_pPanel->GetReader("file://")) != NULL)
								pReader->Remove(tSelectionCopy, false);
					}
				}

				if (bUseGlobalReaderCtl == true)
				{
					pReader->Destroy();
				}
			}
		}
		Refresh();
	}
	else if (state == CLIP_CUT)
	{
		if (_tSelection.Size() > 0)
		{
			g_Log.Write("PasteClip CLIP_CUT Start - [%d] [%s] [%s] !!!", 
								_tSelection.Size(),
								_tSelection[0]->sType.c_str(), 
								_pPanel->GetReader()->GetInitType().c_str());

			if (_tSelection[0]->sType == "file://" && _pPanel->GetReader()->GetInitType() == "file://")
			{
				// sType이 file 이면 그냥 옮긴다.
				_pPanel->GetReader()->Move(_tSelection);
			}
			else
			{
				bool	bUseGlobalReaderCtl = false;
	
				// 패널이 두개이기 때문에 mainframe 의 panel에서 Type에 맞는 데이터를 읽어 온다.
				Reader* pReader = g_tMainFrame.GetPanel(0)->GetReader(_tSelection[0]->sType);
				if (pReader == NULL || pReader->GetInitType() != _tSelection[0]->sType)
				{
					pReader = g_tMainFrame.GetPanel(1)->GetReader(_tSelection[0]->sType);
					if (pReader == NULL || pReader->GetInitType() != _tSelection[0]->sType)
					{
						// 그래도 없으면 만들서 넣어야 함.
						string::size_type p = _tSelection[0]->sType.find("://");
						string sInitName;

						if (p != string::npos)
							sInitName = _tSelection[0]->sType;
						else
							sInitName = _tSelection[0]->sType.substr(0, p);
	
						// 이때는 g_tReaderCtl로 읽어 들여옴.
						pReader = g_tReaderCtl.Get(sInitName);
						if (pReader == NULL)
						{
							MsgBox(gettext("Error"), gettext("plugin not found."));
							return;
						}

						bUseGlobalReaderCtl = true;
						if (p == string::npos)
							sInitName = _tSelection[0]->sType;
						else
							sInitName = _tSelection[0]->sType.substr(p+3);

						pReader->Init(sInitName);
					}
				}

				Selection	tSelectionMove;

				// /tmp 에 복사하고 나서 옮긴다.
				if (pReader->Copy(_tSelection, "/tmp/", &tSelectionMove))
				{
					Reader*		pCurReader = _pPanel->GetReader();
					if (pCurReader->GetInitType() == "file://")
					{
						// 파일일 경우는 이동한다.
						if (pCurReader->Move(tSelectionMove))
							pCurReader->Remove(_tSelection, false); // 이동하고 원본 삭제
					}
					else
					{	
						// 파일이 아닌 경우는 /tmp에서 다른 곳으로 복사 한 후
						// /tmp에 있는 내용은 지운다.
						if (pCurReader->Paste(tSelectionMove))
							if ((pReader = _pPanel->GetReader("file://")) != NULL)
							{
								pReader->Remove(tSelectionMove, false);	// Tmp 디렉토리 지움.
								pCurReader->Remove(_tSelection, false); // 원본 삭제
							}
					}
				}

				if (bUseGlobalReaderCtl == true)
				{
					pReader->Destroy();
				}
			}	
		}
		Refresh();
	}
}

void	CommandImp::Key_ESC()
{
	if (g_tMainFrame.GetScreenSync())
	{
		g_tMainFrame.SetScreenSync(false);
		return;
	}

	switch (g_tMainFrame.GetActiveViewType())
	{
		case PANEL:
			if (_pPanel->GetReader()->GetReaderName() != "file")
			{
				_pPanel->PluginClose("file");
			}
			else
			{
				CursesDestroy();
				cbreak();
				noecho();
				getch();
				CursesInit();
			}
			g_tMainFrame.Refresh(false);
			break;

		case MCD:
			g_tMainFrame.SetActiveViewType(PANEL);
			Refresh();
			break;
	}
}

void	CommandImp::Shell()
{
	switch (g_tMainFrame.GetActiveViewType())
	{
		case PANEL:
			if (_pPanel->GetReader()->GetInitType() == "file://")
				g_tMainFrame.CmdShell(true);
			else
				MsgBox("Error", "shell is local file system use only.");
			break;
	}
}

void	CommandImp::Rename()
{
	switch(g_tMainFrame.GetActiveViewType())
	{
		case MCD:
		{
			File 	tFile;
			string sBefPath = _pPanel->GetPath();
			tFile = _pMcd->GetCurFile();
			_pPanel->GetReader()->Rename(&tFile);
			_pPanel->Read(sBefPath);
			_pMcd->SubDirOneSearch();
			break;
		}
		case PANEL:
			_pPanel->GetReader()->Rename(_pPanel->GetCurFile());
			break;
	}
	Refresh();
}

void	CommandImp::Mkdir()
{
	switch(g_tMainFrame.GetActiveViewType())
	{
		case MCD:
		{
			string sBefPath = _pPanel->GetPath();
			_pPanel->Read(_pMcd->GetCurName());
			_pPanel->GetReader()->Mkdir(); 
			_pPanel->Read(sBefPath);
			_pMcd->SubDirOneSearch();
			break;
		}
		case PANEL:
			_pPanel->GetReader()->Mkdir(); 
			break;
	}
	Refresh();
}

void CommandImp::GoRoot()
{ 
	switch(g_tMainFrame.GetActiveViewType())
	{
		case MCD:
		{
			_pMcd->SetCur("/");
			break;
		}
		case PANEL:
			_pPanel->Read("/");
			break;
	}
	if (g_tMainFrame.GetScreenSync()) ViewSync();
}

void CommandImp::GoHome() 
{ 
	switch(g_tMainFrame.GetActiveViewType())
	{
		case MCD:
		{
			string sBefPath = _pPanel->GetPath();
			_pPanel->Read("~");
			string sPath =_pPanel->GetPath();
			_pPanel->Read(sBefPath);
			_pMcd->SetCur(sPath);
			break;
		}
		case PANEL:
			_pPanel->Read("~");
			break;
	}
	if (g_tMainFrame.GetScreenSync()) ViewSync();
}

void CommandImp::GoParent()
{ 
	switch(g_tMainFrame.GetActiveViewType())
	{
		case MCD:
		{
			_pMcd->Key_Left();
			break;
		}
		case PANEL:
			_pPanel->Read("..");
			break;
	}
	
	if (g_tMainFrame.GetScreenSync()) ViewSync();
}

void CommandImp::BoxCodeChange()
{
	g_tMainFrame.LineCodeChange();
}

void CommandImp::HiddenFileView()
{
	switch(g_tMainFrame.GetActiveViewType())
	{
		case MCD:
			_pMcd->SetHiddenView(!_pMcd->GetHiddenView());
			break;
		
		case PANEL:
			_pPanel->SetHiddenView(!_pPanel->GetHiddenView());
			break;
	}
}

// sort
void    CommandImp::SortChange()
{
	if (g_tMainFrame.GetActiveViewType() != PANEL) return;
	_pPanel->_nSort++;
	if (_pPanel->_nSort == 5) _pPanel->_nSort=0;
	switch(_pPanel->_nSort)
	{
		case 0: SortName();    break;
		case 1: SortExt();    break;
		case 2: SortSize();    break;
		case 3: SortTime(); break;
		case 4: SortColor(); break;
	}
}

void CommandImp::SortName()
{
	if (g_tMainFrame.GetActiveViewType() != PANEL) return;
	_pPanel->SetSortMethod(SORT_NAME);
	_pPanel->Sort();
}

void CommandImp::SortExt()
{
	if (g_tMainFrame.GetActiveViewType() != PANEL) return;
	_pPanel->SetSortMethod(SORT_EXT);
	_pPanel->Sort();
}

void CommandImp::SortSize()
{
	if (g_tMainFrame.GetActiveViewType() != PANEL) return;
	_pPanel->SetSortMethod(SORT_SIZE);
	_pPanel->Sort();
}

void CommandImp::SortTime()
{
	if (g_tMainFrame.GetActiveViewType() != PANEL) return;
	_pPanel->SetSortMethod(SORT_TIME);
	_pPanel->Sort();
}

void CommandImp::SortColor()
{
	if (g_tMainFrame.GetActiveViewType() != PANEL) return;
	_pPanel->SetSortMethod(SORT_COLOR);
	_pPanel->Sort();
}

void CommandImp::ColumnAuto() 
{ 
	if (g_tMainFrame.GetActiveViewType() == PANEL)
		_pPanel->SetViewColumn(0);
}

void CommandImp::Column1()
{ 
	if (g_tMainFrame.GetActiveViewType() == PANEL)
		_pPanel->SetViewColumn(1);
}

void CommandImp::Column2()
{ 
	if (g_tMainFrame.GetActiveViewType() == PANEL)
		_pPanel->SetViewColumn(2);
}

void CommandImp::Column3()
{ 
	if (g_tMainFrame.GetActiveViewType() == PANEL)
		_pPanel->SetViewColumn(3);
}

void CommandImp::Column4()
{ 
	if (g_tMainFrame.GetActiveViewType() == PANEL)
		_pPanel->SetViewColumn(4);
}

