#ifndef __MLS_COMMANDIMP_H__
#define __MLS_COMMANDIMP_H__

#include "define.h"
#include "mlslocale.h"
#include "mlscfgload.h"
#include "ncurses_panel.h"
#include "ncurses_mcd.h"
#include "command.h"
#include "selection.h"

// PluginLoader 때문.
#include "vfsplugin.h"
#include "readerctl.h"

namespace MLS {

class CommandImp:public Configurable
{
private:
	NCurses_Panel*			_pPanel;
	NCurses_Mcd*			_pMcd;
	Selection				_tSelection;
	ClipBoard				_tClipboard;
	vector<NCurses_Panel*>	_pCurPanel;

public:
	CommandImp() : _pPanel(NULL), _pMcd(NULL)
	{
		UpdateConfig();

		// 이것은 Copy&Paste를 위해서 따로 쓰이는 Reader이다.
		// 여기에서 한번만 로드
		PluginLoader(&g_tReaderCtl);
	}

	~CommandImp() 					{ SaveConfig(); }

	void SetPanel(NCurses_Panel* p)	{ _pPanel = p; }
	void SetMcd(NCurses_Mcd* p)		{ _pMcd = p; }
	NCurses_Mcd* 	GetMcd()		{ return _pMcd; }
	
	NCurses_Panel* 	GetPanel()		{ return _pPanel; }

	// IConfigurable
	void UpdateConfig();
	void SaveConfig();

	int  	Run(const std::string &, bool = false, bool = false);
	int		ParseAndRun(const string &p, bool = false);

	void	GoRoot();
	void	GoHome();
	void	GoParent();

	void	Key_Enter();
	void	Key_Left();
	void	Key_Right();
	void	Key_Up();
	void	Key_Down();
	void	Key_Home();
	void	Key_End();
	void	Key_PageUp();
	void	Key_PageDown();
	void	Key_Select()		{ _pPanel->Key_Select();	}

	void	SelectAll()		{ _pPanel->SelectAll();		}
	void	SelectInvert() 	{ _pPanel->SelectInvert();	}

	void	Key_ESC();
	void	Mcd();

	void	Refresh();

	void	ArchiveFileView();
	void	ArchiveFileMake();
	void	FtpConn();
	void	FtpClose();

	void	View();
	void	Editor();
	void	ExtEditor();
	void	Menu();
	void	EditorChoice(bool bReadOnly, const string& sEditorCmd = "");

	void	Quit();	
	void	Split();
	void	NextWindow();
	void	SplitViewSync();
	void	ViewSync();

	void	Remove();
	void	CopyClip();
	void	PasteClip();
	void	CutClip();

	void	Rename();
	void	Mkdir();

	void	Shell();

	void	McdSubDirOneSearch()	{	_pMcd->SubDirOneSearch();	}
	void	McdSubDirAllSearch()	{	_pMcd->SubDirAllSearch();	}
	void	McdSubDirClear() 		{	_pMcd->SubDirClear(); }

	void 	BoxCodeChange();
	void 	LangChange()
	{
		if (e_nCurLang == US)
			Set_Locale(AUTO);
		else
			Set_Locale(US);
	}

	void	HiddenFileView();

	// sort
    void    SortChange();
    void	SortName();
	void	SortExt();
	void	SortSize();
	void	SortTime();
	void	SortColor();
	void	ColumnAuto();
	void	Column1();
	void	Column2();
	void	Column3();
	void	Column4();
}; // class CommandImp

}; // namespace

#endif
