#include "dialogbox.h"

using namespace MLSUTIL;
using namespace MLS;

// YNBox

Curses_YNBox::
Curses_YNBox(const string& sTitle, const string& sMsg, bool bYes): Form()
{
	SetTitle(sTitle);
	_tButtonOk = Button(gettext("Yes"), 10);
	_tButtonCancel = Button(gettext("No"), 10);

	_tLabel	 = Label(sMsg);

	fontcolor = g_tColorCfg.GetColorEntry("Dialog").font;
	backcolor = g_tColorCfg.GetColorEntry("Func").back;
	_tButtonOk.backcolor = backcolor; _tButtonOk.fontcolor = fontcolor;
	_tButtonCancel.backcolor = backcolor; _tButtonCancel.fontcolor = fontcolor;
	_tLabel.backcolor = backcolor; _tLabel.fontcolor = fontcolor;

	_bYes = bYes;
}

void	Curses_YNBox::Resize()
{
	if (width < 34) width = 34;
}

void	Curses_YNBox::Draw()
{
	g_Log.Write("Form height [%d] width [%d]", height, width);
	
	_tButtonOk.SetForm((Form*)this);
	_tButtonCancel.SetForm((Form*)this);
	_tLabel.SetForm((Form*)this);

	_tButtonOk.Move(height - 2, (width/2)-12);
	_tButtonCancel.Move(height - 2, (width/2)+3);
	_tLabel.Move(3, MIDDLE);

	_tButtonOk.SetTxtAlign(MIDDLE);
	_tButtonCancel.SetTxtAlign(MIDDLE);
	_tLabel.SetTxtAlign(MIDDLE);

	_tButtonOk.Show();
	_tButtonCancel.Show();
	_tLabel.Show();

	if (_bYes == true)
	{
		_tButtonOk.SetFocus(true);
		_tButtonCancel.SetFocus(false);
	}
	else
	{
		_tButtonOk.SetFocus(false);
		_tButtonCancel.SetFocus(true);
	}
}

bool	Curses_YNBox::MouseEvent(int Y, int X, mmask_t bstate)
{
	if (_tButtonOk.AreaChk(Y, X))
	{
		_tButtonOk.SetFocus(true);
		_bYes = true;
		if (bstate & BUTTON1_DOUBLE_CLICKED) _bExit = true;
	}

	if (_tButtonCancel.AreaChk(Y, X))
	{
		_tButtonCancel.SetFocus(true);
		_bYes = false;
		if (bstate & BUTTON1_DOUBLE_CLICKED) _bExit = true;
	}

	return false;
}

void	Curses_YNBox::Execute(KeyInfo&	tKeyInfo)
{
	g_Log.Write("Key [%d]", (int)tKeyInfo);

	switch((int)tKeyInfo)
	{
		case KEY_ESC:
		case KEY_ENTER:
		case 13:
			_bExit = true;
			break;

		case KEY_LEFT:
		case KEY_RIGHT:
		case KEY_DOWN:
		case KEY_UP:
		case KEY_TAB:
			_bYes = !_bYes;
			break;
	}
}

// Select Box

void	Curses_SelectBox::Clear()
{
	Button*		pButton = NULL;
	for (int n = 0; n<_vButtonList.size(); n++)
	{
		pButton = _vButtonList[n];
		delete pButton;
	}
}

void	Curses_SelectBox::SetSelData(vector<string>	vData)
{
	Button*		pButton = NULL;

	Clear();

	_nStrMaxLength = width - 4;
	for (int n = 0; n<vData.size(); n++)
	{
		pButton = new Button(vData[n]);

		if (_nStrMaxLength < scrstrlen(vData[n])) _nStrMaxLength = scrstrlen(vData[n]);

		_vButtonList.push_back(pButton);	
	}	
	_vData = vData;
}

void	Curses_SelectBox::Draw()
{
	Button*	pButton = NULL;

	for (int n = 0; n<_vButtonList.size(); n++)
	{
		pButton = _vButtonList[n];
		pButton->SetForm((Form*)this);
		pButton->width = _nStrMaxLength;
		pButton->Move(3+n, MIDDLE);
		pButton->SetTxtAlign(LEFT);
		pButton->Show();

		if (_nCur == n)
			pButton->SetFocus(true);
		else
			pButton->SetFocus(false);
	}
}

bool	Curses_SelectBox::MouseEvent(int Y, int X, mmask_t bstate)
{
	Button*	pButton = NULL;

	for (int n = 0; n<_vButtonList.size(); n++)
	{
		pButton = _vButtonList[n];

		if (pButton->AreaChk(Y, X))
		{
			_nCur = n;
			if (bstate & BUTTON1_DOUBLE_CLICKED) _bExit = true;
			return true;
		}
	}
	return false;
}

void	Curses_SelectBox::Execute(KeyInfo&	tKeyInfo)
{
	switch((int)tKeyInfo)
	{
		case KEY_ESC:
		case KEY_ENTER:
		case 13:
			g_Log.Write("Key ENTER [%d]", (int)tKeyInfo);
			_bExit = true;
			break;

		case KEY_TAB:
		case KEY_UP:
		case KEY_LEFT:
			_nCur--;
			if (_nCur < 0) _nCur = _vData.size() - 1;
			break;

		case KEY_DOWN:
		case KEY_RIGHT:
			_nCur++;
			if (_vData.size() <= _nCur) _nCur = 0;
			break;
	}
}

// Input Box

void	Curses_InputBox::Draw()
{
	_tInputBox.SetSize(1, width - 4);

	_tInputBox.backcolor = 4;
	_tButtonOk.SetForm((Form*)this);
	_tButtonCancel.SetForm((Form*)this);
	_tInputBox.SetForm((Form*)this);

	_tButtonOk.Move(height - 2, (width/2)-12);
	_tButtonCancel.Move(height - 2, (width/2)+3);
	_tInputBox.Move(3, MIDDLE);

	_tButtonOk.SetTxtAlign(MIDDLE);
	_tButtonCancel.SetTxtAlign(MIDDLE);
	
	_tButtonOk.Show();
	_tButtonCancel.Show();
	_tInputBox.Show();

	if (_bStat == false)
	{
		if (_bYes == true)
		{
			_tButtonOk.SetFocus(true);
			_tButtonCancel.SetFocus(false);
		}
		else
		{
			_tButtonOk.SetFocus(false);
			_tButtonCancel.SetFocus(true);
		}
	}
	else
	{
		_tButtonOk.SetFocus(false);
		_tButtonCancel.SetFocus(false);
		_tInputBox.SetFocus();
	}
}

bool	Curses_InputBox::MouseEvent(int Y, int X, mmask_t bstate)
{
	if (_tButtonOk.AreaChk(Y, X))
	{
		_tButtonOk.SetFocus(true);
		_bYes = true;
		if (bstate & BUTTON1_DOUBLE_CLICKED) _bExit = true;
		return true;
	}

	if (_tButtonCancel.AreaChk(Y, X))
	{
		_tButtonCancel.SetFocus(true);
		_bYes = false;
		if (bstate & BUTTON1_DOUBLE_CLICKED) _bExit = true;
		return true;
	}

	if (_tInputBox.AreaChk(Y, X))
	{
		_bStat = true;
		return true;
	}
	return false;
}

void	Curses_InputBox::Execute(KeyInfo& tKeyInfo)
{
	int nKey = (int)tKeyInfo;

	if (_bStat == true)
	{
		nKey = _tInputBox.DataInput(tKeyInfo);
		
		switch(nKey)
		{
			case KEY_MOUSE:
				MouseProc();
				break;
			case KEY_ENTER:
			case 13:
				_bExit = true;
			case KEY_TAB:
				_bYes = true;
				_bStat = false;
				break;
			case KEY_ESC:
				_bYes = false;
				_bStat = false;
				break;
			default:
				_bStat = false;
		}
	}
	else
	{
		switch(nKey)
		{
			case KEY_TAB:
				if (_bYes == false)
					_bStat = true;
				else
					_bYes = !_bYes;
				break;
			case KEY_LEFT:
			case KEY_RIGHT:
			case KEY_UP:
			case KEY_DOWN:
				_bYes = !_bYes;
				break;
			case KEY_ESC:
			case KEY_ENTER:
			case 13:
				_bExit = true;
				break;
		}
	}
}
