#ifndef  __DIALOGBOX_H__
#define  __DIALOGBOX_H__

#include "define.h"
#include "strutil.h"
#include "dialog.h"
#include "colorcfgload.h"

using namespace MLSUTIL;

namespace MLS
{

class Curses_MsgBox:public Form
{
public:
	Curses_MsgBox(const string& sTitle, const string& sMsg): Form()
	{
		SetTitle(sTitle);
		_tButton = Button(gettext("Ok"), 12);
		_tLabel	 = Label(sMsg);
		fontcolor = g_tColorCfg.GetColorEntry("Dialog").font;
		backcolor = g_tColorCfg.GetColorEntry("Dialog").back;
		_tButton.backcolor = backcolor; _tButton.fontcolor = fontcolor;
		_tLabel.backcolor = backcolor; _tLabel.fontcolor = fontcolor;
	}

	Curses_MsgBox(): Form()
	{
		_tButton = Button(gettext("Ok"), 12);
		_tLabel	 = Label("");

		fontcolor = g_tColorCfg.GetColorEntry("Dialog").font;
		backcolor = g_tColorCfg.GetColorEntry("Dialog").back;
		_tButton.backcolor = backcolor; _tButton.fontcolor = fontcolor;
		_tLabel.backcolor = backcolor; _tLabel.fontcolor = fontcolor;
	}

	void	SetMsg(const string& str)	
	{ 
		_tLabel.SetMsg(str); 
		fontcolor = g_tColorCfg.GetColorEntry("Dialog").font;
		backcolor = g_tColorCfg.GetColorEntry("Dialog").back;
		_tButton.backcolor = backcolor; _tButton.fontcolor = fontcolor;
		_tLabel.backcolor = backcolor; _tLabel.fontcolor = fontcolor;
	}

protected:
	void	Draw()
	{
		g_Log.Write("Form height [%d] width [%d]", height, width);
		_tButton.SetForm((Form*)this);
		_tLabel.SetForm((Form*)this);

		_tButton.SetTxtAlign(MIDDLE);
		_tButton.Move(height - 2, MIDDLE);
		_tButton.Show();

		_tLabel.SetTxtAlign(MIDDLE);
		_tLabel.Move(3, MIDDLE);
		_tLabel.Show();

		_tButton.SetFocus(true);
	}

	bool	MouseEvent(int Y, int X, mmask_t bstate)
	{
		if (_tButton.AreaChk(Y, X) && (bstate & BUTTON1_CLICKED))
		{
			_tButton.SetFocus(true);
			_bExit = true;
			g_Log << "Mouse Event True !!!";
			return true;
		}
		return false;
	}

	void 	Execute(KeyInfo& tKeyInfo)
	{
		switch ((int)tKeyInfo)
		{
			case KEY_ESC:
			case KEY_ENTER:
			case 13:
			case ' ':
				_bExit = true;
				break;
		}
	}

private:
	Button	_tButton;
	Label	_tLabel;
};

class Curses_YNBox:public Form
{
public:
	Curses_YNBox(const string& sTitle, const string& sMsg, bool bYes = false);

	bool	GetData() { return _bYes; }

protected:
	void	Resize();
	void	Draw();
	bool	MouseEvent(int Y, int X, mmask_t bstate);
	void	Execute(KeyInfo&	tKeyInfo);

private:
	Button			_tButtonOk;
	Button			_tButtonCancel;
	Label			_tLabel;
	bool			_bYes;
};

class Curses_SelectBox:public Form
{
public:
	Curses_SelectBox(const string& sTitle, int nDef = 0): Form()
	{
		SetTitle(sTitle);
		_nCur = nDef;
	}

	~Curses_SelectBox()
	{
		Clear();
	}

	void	Clear();
	void	SetSelData(vector<string>	vData);
	int		GetNum()	{ return _nCur; }

protected:
	void	Draw();
	bool	MouseEvent(int Y, int X, mmask_t bstate);
	void	Execute(KeyInfo&	tKeyInfo);

private:
	vector<Button*>	_vButtonList;
	vector<string>	_vData;
	int				_nCur;
	int				_nStrMaxLength;
};

class Curses_InputBox:public Form
{
public:
	Curses_InputBox(const string& sTitle, const string& sStr, bool bPasswd = false): Form()
	{
		SetTitle(sTitle);
		_tInputBox = Input(sStr);
		_tInputBox.SetPasswdType(bPasswd);
		_tButtonOk = Button(gettext("Ok"), 12);
		_tButtonCancel = Button(gettext("Cancel"), 12);
		_bStat = true;
		_bYes = true;

		fontcolor = g_tColorCfg.GetColorEntry("Dialog").font;
		backcolor = g_tColorCfg.GetColorEntry("Func").back;
		_tButtonOk.backcolor = backcolor; 	_tButtonCancel.backcolor = backcolor;
		_tButtonOk.fontcolor = fontcolor;	_tButtonCancel.fontcolor = fontcolor;
		_tInputBox.backcolor = 0;
	}

	~Curses_InputBox() { }

	bool	GetStat() { return _bYes; }

	string	GetStr()
	{
		return _tInputBox.GetStr();
	}

protected:
	void	Draw();
	bool	MouseEvent(int Y, int X, mmask_t bstate);
	void	Execute(KeyInfo& tKeyInfo);

private:
	Button			_tButtonOk;
	Button			_tButtonCancel;
	Input			_tInputBox;
	bool			_bStat;
	bool			_bYes;
};

class Curses_ProgressBox:public Form
{
public:
	Curses_ProgressBox(const string& sTitle, const string& sMsg, bool bDouble = false): Form()
	{
		SetTitle(sTitle);
		_sTitle = sTitle; _sMsg = sMsg;	_bDouble = bDouble;
		_bExit = false;

		_tButtonCancel = Button(gettext("Cancel"), 12);
		_tLabel	 = Label(sMsg);

		_tLabelUpLeft = Label("");	_tLabelUpRight = Label("");
		_tLabelDnLeft = Label("");	_tLabelDnRight = Label("");

		fontcolor = g_tColorCfg.GetColorEntry("Dialog").font;
		backcolor = g_tColorCfg.GetColorEntry("Func").back;
		_tLabel.fontcolor = _tLabelUpLeft.fontcolor = _tLabelUpRight.fontcolor = fontcolor;
		_tLabelDnLeft.fontcolor = _tLabelDnRight.fontcolor = fontcolor;
		_tLabel.backcolor = _tLabelUpLeft.backcolor = _tLabelUpRight.backcolor = backcolor;
		_tLabelDnLeft.backcolor = _tLabelDnRight.backcolor = backcolor;
	}

	~Curses_ProgressBox() { }

	bool isExit()	{ return _bExit; }

	void setLeftStr(const string& p)  { _tLabelUpLeft.SetMsg(p);  }
	void setRightStr(const string& p) { _tLabelUpRight.SetMsg(p); _sRightStr = p; }
	void setLeftStr2(const string& p) { _tLabelDnLeft.SetMsg(p);  }
	void setRightStr2(const string& p){ _tLabelDnRight.SetMsg(p); _sRightStr2 = p; }

	void setCount(int nCnt, int nCnt2 = -1)
	{
		_nCnt = nCnt;
		if (nCnt2 != -1) _nCnt2 = nCnt2;
	}

	void	Do()
	{
		Show();

		KeyReader	tKeyReader;
		KeyInfo		tKeyInfo = tKeyReader.Read(stdscr, true);

		switch((int)tKeyInfo)
		{
			case KEY_MOUSE:
				MouseProc();
				break;
			case KEY_RESIZE:
			case KEY_REFRESH:
			{
				Refresh();
				return;
			}
		}

		Execute(tKeyInfo);
	}

protected:
	void	Draw()
	{
		_tProgress1.SetForm((Form*)this);
		_tProgress2.SetForm((Form*)this);
		_tButtonCancel.SetForm((Form*)this);
		_tLabel.SetForm((Form*)this);
		_tLabelUpLeft.SetForm((Form*)this);
		_tLabelUpRight.SetForm((Form*)this);
		_tLabelDnLeft.SetForm((Form*)this);
		_tLabelDnRight.SetForm((Form*)this);

		_tProgress1.width = width - 4;
		_tProgress2.width = width - 4;

		_tButtonCancel.Move(height - 2, MIDDLE);
		_tButtonCancel.SetTxtAlign(MIDDLE);
		_tButtonCancel.Show();

		_tLabel.SetTxtAlign(MIDDLE);
		_tLabel.Move(3, MIDDLE);
		_tLabel.Show();

		_tProgress1.Move(5, MIDDLE);
		_tProgress1.SetPercent(_nCnt);
		_tProgress1.Show();

		setcol(fontcolor, backcolor, GetWin());
		wmove(GetWin(), 4, _tProgress1.x);
		whline(GetWin(), ' ', _tProgress1.width);

		_tLabelUpLeft.SetTxtAlign(LEFT);
		_tLabelUpLeft.Move(4, _tProgress1.x);
		_tLabelUpLeft.Show();

		_tLabelUpRight.SetTxtAlign(LEFT);
		_tLabelUpRight.Move(4, _tProgress1.x + (_tProgress1.width - scrstrlen(_sRightStr)));
		_tLabelUpRight.Show();

		if (_bDouble)
		{
			_tProgress2.Move(7, MIDDLE);
			_tProgress2.SetPercent(_nCnt2);
			_tProgress2.Show();

			setcol(fontcolor, backcolor, GetWin());
			wmove(GetWin(), 8, _tProgress2.x);
			whline(GetWin(), ' ', _tProgress2.width);

			_tLabelUpLeft.SetTxtAlign(LEFT);
			_tLabelDnLeft.Move(8, _tProgress2.x);
			_tLabelDnLeft.Show();
			_tLabelDnRight.SetTxtAlign(LEFT);
			_tLabelDnRight.Move(8, _tProgress2.x + _tProgress2.width - scrstrlen(_sRightStr2));
			_tLabelDnRight.Show();
		}

		_tButtonCancel.SetFocus(true);
	}

	bool	MouseEvent(int Y, int X, mmask_t bstate)
	{
		if (_tButtonCancel.AreaChk(Y, X) && (bstate & BUTTON1_DOUBLE_CLICKED))
		{
			_tButtonCancel.SetFocus(true);
			_bExit = true;
			g_Log << "Mouse Event True !!!";
			return true;
		}
		return false;
	}

	void 	Execute(KeyInfo& tKeyInfo)
	{
		switch ((int)tKeyInfo)
		{
			case KEY_ESC:
			case KEY_ENTER:
			case ' ':
				_bExit = true;
				break;
		}
	}

private:
	Button		_tButtonCancel;
	Label		_tLabel;
	Label		_tLabelUpLeft, _tLabelUpRight;
	Label		_tLabelDnLeft, _tLabelDnRight;

	string		_sRightStr;
	string		_sRightStr2;

	Progress	_tProgress1, _tProgress2;
	
	bool		_bDouble;		///< Progress가 위아래 두개면 true
	string		_sTitle, _sMsg;

	bool		_bExit;

	int 		_nCnt;  		///< 카운터 기준 (% 단위)
	int 		_nCnt2;  		///< 카운터 기준 (% 단위)
};

class Curses_ScrollBar:public Form
{
	
};

class Curses_CheckBox:public Form
{
	
};

};

#endif
