#include "dialogbox.h"
#include "mlsdialog.h"

using namespace MLS;

namespace MLSUTIL
{

void	MsgBox(const string& sTitle, const string& sMsg)
{
	Curses_MsgBox	tMsgBox(sTitle, sMsg);
	int size = scrstrlen(sMsg) > 20 ? scrstrlen(sMsg) : 20;
	if (size > COLS-10) size = COLS-10;
	tMsgBox.Size(7, size + 6);
	tMsgBox.Move(MIDDLE, MIDDLE);
	tMsgBox.Do();
}

bool	YNBox(const string& sTitle, const string& sMsg, bool bYes)
{
	Curses_YNBox	tYNBox(sTitle, sMsg, bYes);
	int size = scrstrlen(sMsg) > 20 ? scrstrlen(sMsg) : 20;
	if (size > COLS-10) size = COLS-10;
	tYNBox.Size(7, size+4);
	tYNBox.Move(MIDDLE, MIDDLE);
	tYNBox.Do();
	return tYNBox.GetData();
}

static	Curses_MsgBox	s_tMsgBox("", "");

void*	MsgWaitBox(const string& sTitle, const string& sMsg)
{
	s_tMsgBox.SetTitle(sTitle);
	s_tMsgBox.SetMsg(sMsg);
	int size = scrstrlen(sMsg) > 20 ? scrstrlen(sMsg) : 20;
	if (size > COLS-10) size = COLS-10;
	s_tMsgBox.Size(7, size+6);
	s_tMsgBox.Move(MIDDLE, MIDDLE);
	s_tMsgBox.Show();
	return (void*)&s_tMsgBox;
}

void	MsgWaitEnd(void* pWait)
{
	s_tMsgBox.Close();
}

int		InputBox(const string& sTitle, string &sInputStr, bool bPasswd)
{
	Curses_InputBox		tInputBox(sTitle, sInputStr, bPasswd);
	int size = scrstrlen(sTitle) > 40 ? scrstrlen(sTitle) : 40;
	if (size > COLS-10) size = COLS-10;
	tInputBox.Size(7, size+6);
	tInputBox.Move(MIDDLE, MIDDLE);
	tInputBox.Do();
	if (tInputBox.GetStat() == false) return ERROR;
	sInputStr = tInputBox.GetStr();
	return SUCCESS;
}

int SelectBox(	const string& sTitle,
				vector<string>& vMsgStr,
				int n)
{
	Curses_SelectBox	tSelectBox(sTitle, n);
	tSelectBox.Size(vMsgStr.size()+4, 30);
	tSelectBox.SetSelData(vMsgStr);	
	tSelectBox.Move(MIDDLE, MIDDLE);
	tSelectBox.Do();
	return tSelectBox.GetNum();
}

int	GetChar(bool bNoDelay)
{
	// delay 처리
	if (bNoDelay)	nodelay(stdscr, TRUE);
	int n = getch();
	if (bNoDelay)	nodelay(stdscr, FALSE);
	return n;
}

ProgressBox::ProgressBox(const string& sTitle, const string& sMsg, bool bDouble)
{
	_bDouble = bDouble;
	_sTitle = sTitle;
	_sMsg = sMsg;

	Curses_ProgressBox* pProgress = new Curses_ProgressBox(sTitle, sMsg, bDouble);
	_pVoidTmp = (void*)pProgress;
	int size = scrstrlen(sTitle) > 50 ? scrstrlen(sTitle) : 50;
	if (!bDouble)
		pProgress->Size(9, size);
	else
		pProgress->Size(11, size);
	pProgress->Move(MIDDLE, MIDDLE);
}

ProgressBox::~ProgressBox()
{
	Curses_ProgressBox* pProgress = (Curses_ProgressBox*)_pVoidTmp;
	pProgress->Close();
	delete pProgress;
}

void ProgressBox::show()
{
	Curses_ProgressBox* pProgress = (Curses_ProgressBox*)_pVoidTmp;
	
	if (_sLeftStr.size()) pProgress->setLeftStr(_sLeftStr);
	if (_sRightStr.size()) pProgress->setRightStr(_sRightStr);
	if (_sLeftStr2.size()) pProgress->setLeftStr2(_sLeftStr2);
	if (_sRightStr2.size()) pProgress->setRightStr2(_sRightStr2);
	
	pProgress->setCount(_nCnt, _nCnt2);
	pProgress->Do();
}

void ProgressBox::redraw()
{
	show();
}

bool ProgressBox::isExit()
{
	Curses_ProgressBox* pProgress = (Curses_ProgressBox*)_pVoidTmp;
	return pProgress->isExit();
}

};
