/******************************************************************************
 *   Copyright (C) 2005 by la9527                                             *
 *                                                                            *
 *  This program is free software; you can redistribute it and/or modify      * 
 *  it under the terms of the GNU General Public License as published by      *
 *  the Free Software Foundation; either version 2 of the License, or         *
 *  (at your option) any later version.                                       *
 *                                                                            *
 *  This program is distributed in the hope that it will be useful,           *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU General Public License for more details.                              *
 *                                                                            *
 *  You should have received a copy of the GNU General Public License         *
 *  along with this program; if not, write to the Free Software               *
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.*
 ******************************************************************************/

#ifndef __NCURSES_MCD_H__
#define __NCURSES_MCD_H__

#include "define.h"
#include "mcd.h"
#include "strutil.h"
#include "dialog.h"
#include "mlsdialog.h"
#include "colorcfgload.h"
#include "mpool.h"

using namespace MLSUTIL;

namespace MLS
{

class McdDirButton:public Position
{
private:
	Dir*	_pNode;
	bool	_bSelect;
	bool	_bShowCheck;

protected:
	void	Draw()
	{
		WINDOW*	pWin = _pForm->GetWin();
		if (_pNode == NULL) return;

		ColorEntry tMCD 	= g_tColorCfg.GetColorEntry("MCD");
		ColorEntry tMCDLine = g_tColorCfg.GetColorEntry("MCDLine");

		setcol(tMCD, pWin);
		wmove(pWin, y, x);
		whline(pWin, ' ', width);
		
		if (_pNode->tFile.sFullName == "/")
		{
			if (_bSelect)
				setrcol(tMCD, pWin);
			else
				setcol(tMCD, pWin);
			wprintw(pWin, "/");
		}
		else
		{
			if (_bSelect)
			{
				setrcol(tMCD, pWin);
				g_Log.Write("Selected [%s]", _pNode->tFile.sName.c_str());
				if (_pNode->tFile.sName.size() <= 12)
					wprintw(pWin, "%-12s", _pNode->tFile.sName.c_str());
				else
					wprintw(pWin, "%-11.11s~", _pNode->tFile.sName.c_str());
			}
			else
			{
				g_Log.Write("Mcd Data [%s]", _pNode->tFile.sName.c_str());

				if (_pNode->tFile.sName.size() <= 12)
				{
					setcol(tMCD, pWin);
					wprintw(pWin, "%s ", _pNode->tFile.sName.c_str());
					
					if (!_pNode->vNode.empty())
					{
						setcol(tMCDLine, pWin);
						whline(pWin, HLINE, 12-_pNode->tFile.sName.size());
					}
				}
				else 
				{
					setcol(tMCD, pWin);
					wprintw(pWin, "%-11.11s~", _pNode->tFile.sName.c_str());
				}
			}
		}

		wnoutrefresh(pWin);
	}

public:
	McdDirButton()
	{
		_pNode = NULL; _bSelect = false; _bShowCheck = false;
	}	

	void	SetDir(Dir* pDir, bool bSel = false) { _pNode = pDir; _bSelect = bSel; _bShowCheck = true; }
	bool	GetShowCheck() { return _bShowCheck; }
	Dir*	GetNode() { return _pNode; }
};

class NCurses_Mcd:public Mcd, public Form
{
private:
	vector<McdDirButton*>		_vDirButtonList;
	MemoryPool<McdDirButton>	_tMemPoolDirButton;
	int							_nBefMemSize;
	int							_nRowSize;

protected:
	void	DrawInit();
	void 	Draw();

	void	Refresh()
	{
		werase(_pWin);
		Draw();
	}

public:
	bool	_bFocus;

	NCurses_Mcd(const string& sDir = ""):Mcd(sDir)
	{
		_nRowSize = 0;
		_nBefMemSize = 0;
		_bNoOutRefresh = true; // Refresh 할때 doupdate 하지 않는다.
		_bNoViewUpdate = true; // 업데이트 하지 않는다.
		_bNotDrawBox = true; // 박스를 그리지 않는다.
		_bFocus = false;
	}

	bool	MouseEvent(int Y, int X, mmask_t bstate)
	{
		McdDirButton* pDirButton = NULL;

		int nY = Y - y; int nX = X - x;
		for (int n = 0; n<_vDirButtonList.size(); n++)
		{
			pDirButton = _vDirButtonList[n];
			if (pDirButton->GetShowCheck() == true)
			{
				if (pDirButton->AreaChk(nY, nX))
				{
					g_Log.Write("MouseEvent Y [%d] X [%d]", Y, X);
					if (bstate & BUTTON1_CLICKED) 
					{
						SetCur(pDirButton->GetNode()->tFile.sFullName);
					}
					else if (bstate & BUTTON1_DOUBLE_CLICKED) 
					{
						SetCur(pDirButton->GetNode()->tFile.sFullName);
						return true;
					}
					break;
				}
			
				if (pDirButton->y == nY && pDirButton->x == nX+1)
				{
					if (bstate & BUTTON1_CLICKED) 
					{
						if (pDirButton->GetNode()->bCheck == false)
						{
							SetCur(pDirButton->GetNode()->tFile.sFullName);
							SubDirOneSearch();
						}
						else
						{
							SetCur(pDirButton->GetNode()->tFile.sFullName);
							SubDirClear();
						}
					}
					break;
				}
			}
		}
		return false;
	}

	void	Key_PageDown()
	{
		int nDepth = (*_pCur)->nDepth;
		int nFindRow = (*_pCur)->nRow + height - 3;
		Dir* pNode = GetDir_RowArea(nFindRow, nDepth);
		
		if (pNode)
		{
			g_Log.Write("pNode [%d]", pNode->nRow);
			g_Log.Write("pNode [%d]", pNode->tFile.sFullName.c_str());
			SetCur(pNode->tFile.sFullName);
		}
	}

	void	Key_PageUp()
	{
		int nDepth = (*_pCur)->nDepth;
		int nFindRow = (*_pCur)->nRow - height + 3;
		Dir* pNode = GetDir_RowArea(nFindRow, nDepth);
		
		if (pNode)
		{
			g_Log.Write("pNode [%d]", pNode->nRow);
			g_Log.Write("pNode [%d]", pNode->tFile.sFullName.c_str());
			SetCur(pNode->tFile.sFullName);
		}
	}
};

};

#endif
